//=============================================================================
// TCOOverlay.
// Description: If not client-only then draws itself only when the server allows it
//					depending on if the server-side lists checks have passed.
//					If client-only then it does the same but only client-side.
//					(See TCOMutator's notes)
// 
// Date: 2021
// ©copyright Francesco Biscazzo. All rights reserved.
//=============================================================================
class TCOOverlay extends Decoration;

//#exec TEXTURE IMPORT NAME=transparencyGridTexture FILE=Textures\transparency_grid.bmp GROUP=Skins
//#exec TEXTURE IMPORT NAME=blackTexture FILE=Textures\black.bmp GROUP=Skins
#exec TEXTURE IMPORT NAME=whiteTexture FILE=Textures\white.bmp GROUP=Skins
#exec TEXTURE IMPORT NAME=redTexture FILE=Textures\red.bmp GROUP=Skins
#exec TEXTURE IMPORT NAME=blueTexture FILE=Textures\blue.bmp GROUP=Skins
#exec TEXTURE IMPORT NAME=greenTexture FILE=Textures\green.bmp GROUP=Skins
#exec TEXTURE IMPORT NAME=goldTexture FILE=Textures\gold.bmp GROUP=Skins
#exec TEXTURE IMPORT NAME=whiteRadialTexture FILE=Textures\white_radial.bmp GROUP=Skins
#exec TEXTURE IMPORT NAME=redRadialTexture FILE=Textures\red_radial.bmp GROUP=Skins
#exec TEXTURE IMPORT NAME=blueRadialTexture FILE=Textures\blue_radial.bmp GROUP=Skins
#exec TEXTURE IMPORT NAME=greenRadialTexture FILE=Textures\green_radial.bmp GROUP=Skins
#exec TEXTURE IMPORT NAME=goldRadialTexture FILE=Textures\gold_radial.bmp GROUP=Skins
//#exec LOAD FILE=Textures\Packages\ScriptedTextures.utx PACKAGE=TeamColorOverlay_v1.ScriptedTextures

var TCOMutator mut;
var bool bClientOnly;

var Texture /*transparencyGridTexture, blackTexture, */whiteTexture, redTexture, blueTexture, greenTexture, goldTexture, whiteRadialTexture, redRadialTexture, blueRadialTexture, greenRadialTexture, goldRadialTexture;
var bool bCanBeDrawn;

replication {
	// getMut()? Occam's razor.
	reliable if (role == ROLE_Authority)
		mut;
		
	reliable if (role == ROLE_Authority)
		bCanBeDrawn;
}

simulated function String getColorTextures(out Texture colorTex, out Texture colorRadialTex) {
	local PlayerReplicationInfo pri;
		
	if ((owner != None) && (owner.instigator != None)) {
		if (owner.instigator == mut.localPP) {
			colorTex = whiteTexture;
			colorRadialTex = whiteRadialTexture;
		} else {
			pri = owner.instigator.playerReplicationInfo;
			if (pri != None) {
				if (!mut.bTeamCheck || !mut.localPP.gameReplicationInfo.bTeamGame) {
					colorTex = redTexture;
					colorRadialTex = redRadialTexture;
				} else {
					if (pri.team == 0) {
						colorTex = redTexture;
						colorRadialTex = redRadialTexture;
					} else if (pri.team == 1) {
						colorTex = blueTexture;
						colorRadialTex = blueRadialTexture;
					} else if (pri.team == 2) {
						colorTex = greenTexture;
						colorRadialTex = greenRadialTexture;
					} else if (pri.team == 3) {
						colorTex = goldTexture;
						colorRadialTex = goldRadialTexture;
					}
				}
			} // else based on attitudeToPlayer?
		}
	} // else based on attitudeToPlayer?
	
	//mut.baseModule.processColorTextures(colorTex, colorRadialTex);
	
	if ((colorTex == None) || (colorRadialTex == None))
		//colorTex = transparencyGridTexture;
		mut.debugLog("No overlay texture for:"@owner);
}

simulated function setTextures(Texture colorTex, Texture colorRadialTex) {
	local int i;
	
	texture = colorRadialTex;
	skin = colorTex;
	for (i = 0; i < arrayCount(multiSkins); i++)
		multiSkins[i] = colorTex;
}

simulated function drawOwner(Canvas canvas) {
	local Texture colorTex, colorRadialTex;
	local vector loc, camLocFromCenter;
	local rotator camRotFromCenter;
	local Actor viewActor;
	local float dist, zDist;
	local bool bWireframe;

	if (owner != None) {
		getColorTextures(colorTex, colorRadialTex);
		if ((colorTex != None) && (colorRadialTex != None)) {
			LODBias = mut.overlayLODBias;
		
			class'TCOUtils'.static.calcPlayerViewFromCenter(mut.localPP, viewActor, camLocFromCenter, camRotFromCenter);
			
			loc = owner.location;
			bWireframe = mut.bWireframe && (owner == mut.localPP);
			if (mut.bOutlineOnly && !bWireframe)
				loc += normal(owner.location - camLocFromCenter) * mut.outlineDist;
			setLocation(loc);
			
			setRotation(owner.rotation);
			
			drawType = owner.drawType;
			drawScale = owner.drawScale;
			if ((drawType == DT_Sprite) || (drawType == DT_RopeSprite) || (drawType == DT_VerticalSprite) || (drawType == DT_SpriteAnimOnce)) {
				// Proportionate the UV sizes of the sprite's texture to the drawScale.
				if (owner.texture != None)
					drawScale *= fMax(owner.texture.uSize, owner.texture.vSize) / fMax(colorRadialTex.uSize, colorRadialTex.vSize);
			} else if (drawType == DT_Mesh) {
				mesh = owner.mesh;
				animSequence = owner.animSequence;
				animFrame = owner.animFrame;
				
				// The thickness is based on the distance between the player and the owner and the FOV angle of the player.
				dist = vSize(owner.location - camLocFromCenter);
				// Get the FOVAngle ratio to 170 (standard maximum FOVAngle) then lerp it with the dist and get the ratio to 1300 (an arbitrary distance).
				// Then use that to get the thickness by lerping.
				fatness = fMin(owner.fatness + lerp(fMin(lerp(fMin(mut.localPP.FOVAngle / 170.0, 1), 0, dist) / 1300.0, 1), mut.minOverlayThickness, mut.maxOverlayThickness), 255);
				
				if ((colorRadialTex == None) || !mut.bUseParticles) {
					bParticles = false;
				} else {
					// When bParticles is true the mesh stays untouched, some settings that used to change the look of the mesh now change the look of the particles.
					drawScale = 6.4 / fMax(colorRadialTex.uSize, colorRadialTex.vSize);
					//LODBias = max(LODBias, 9999);
					bParticles = true;
				}
			}
			
			setTextures(colorTex, colorRadialTex);
			
			canvas.drawActor(self, bWireframe, false);
		}
	}
}

simulated event tick(float deltaTime) {
	local bool bTempCanBeDrawn;

	super.tick(deltaTime);
	
	if ((role == ROLE_Authority) || bClientOnly) {
		if ((owner != None) && (owner.bDeleteMe))
			// Sometimes the reference to an object isn't set to None even if the object has been "deleted".
			// So make an attempt to predict when the owner is about to be deleted and set it to None.
			setOwner(None);
		
		if (owner != None)
			if (!mut.isStateBlacklisted(owner.getStateName()) && !mut.isTextureBlacklisted(owner.texture))
				if (owner.instigator != None)
					if (!owner.bHidden && (owner.drawScale != 0) && ((owner.drawType != DT_Mesh) || ((owner.drawType == DT_Mesh) && (owner.mesh != None) && (owner.fatness != 0))))
						//if (mut.baseMoule.canBeDrawn(owner))
						bTempCanBeDrawn = true;
		// Setting bCanBeDrawn to false before the checks may overload the replication's "reliables" queue.
		bCanBeDrawn = bTempCanBeDrawn;
	}
}

//transparencyGridTexture=transparencyGridTexture
//blackTexture=blackTexture
defaultproperties {
	remoteRole=ROLE_SimulatedProxy
	bAlwaysRelevant=True

	bStatic=False
	bStasis=False
	style=STY_Translucent
	bUnlit=True
	bHidden=True
	
	whiteTexture=whiteTexture
	redTexture=redTexture
	blueTexture=blueTexture
	greenTexture=greenTexture
	goldTexture=goldTexture
	whiteRadialTexture=whiteRadialTexture
	redRadialTexture=redRadialTexture
	blueRadialTexture=blueRadialTexture
	greenRadialTexture=greenRadialTexture
	goldRadialTexture=goldRadialTexture
}
